/*
 * Copyright (c) 2015-2021 by inovat
 * innovative systeme - verkehr - tunnel - technik,
 * Dipl.-Ing. H. C. Kniss
 *
 * This file is part of de.bsvrz.kex.tls.osi7.TlsData
 *
 * de.bsvrz.kex.tls.osi7.TlsData is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * de.bsvrz.kex.tls.osi7.TlsData is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with de.bsvrz.kex.tls.osi7.TlsData.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Diese Datei ist Teil von de.bsvrz.kex.tls.osi7.TlsData.
 *
 * de.bsvrz.kex.tls.osi7.TlsData ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 *
 * de.bsvrz.kex.tls.osi7.TlsData wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 *
 * Sie sollten eine Kopie der GNU General Public License zusammen mit diesem
 * Programm erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 *
 * Contact Information:
 * inovat, Dipl.-Ing. H. C. Kniss
 * An der Krautwiese 37
 * D-53783 Eitorf
 * +49 (0)2243 8464 193
 * info@inovat.de
 * www.inovat.de
 */

package de.bsvrz.kex.tls.osi7;

import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.sys.funclib.hexdump.HexDumper;

/**
 * Ermöglicht über ein TlsData Objekt eine komfortablen Zugriff auf TlsByte[]-Daten. Der Zugriff auf die Daten erfolgt
 * gekapselt über die Klassenmethoden. Es lassen sich mehrere Byte Lesen oder Schreiben, wobei die entsprechenden
 * Methoden die Interpretation bzgl. Reihenfolge und Wert (Low-High-ExtraHigh-Byte) vornehmen. Eine Lese- oder
 * Schreiboperation setzt den interen Lesezeiger im um die notwendige Anzahl Bytes weiter, so dass kontinuierlich
 * geschrieben bzw. gelesen werden kann. Wird über das Ende hinaus gelesen, so wird eine {@link
 * ArrayIndexOutOfBoundsException} erzeugt. Das übergebene tlsByte[] wird nicht verändert.
 *
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @author Dipl.-Ing. Hans Christian Kniß (HCK)
 */
public class TlsData {
// ------------------------------------------------------------------------
// VARIABLENDEKLARATIONEN
// ------------------------------------------------------------------------

// Sonstige ---------------------------------------------------------------
	/**
	 * DebugLogger für Debug-Ausgaben
	 */
	private static final Debug debug = Debug.getLogger();
// Private ----------------------------------------------------------------
	/**
	 * Interne Darstellung des byte[]. Das übergebene Bytearray wird nicht verändert.
	 */
	private int[] _tlsData;

	/**
	 * Aktueller LeseIndex innerhalb des Datenblocks.
	 */
	private int _currentReadPosition;

	/**
	 * Aktueller SchreibIndex innerhalb des Datenblocks.
	 */
	private int _currentWritePosition;

	/**
	 * Letzter LeseIndex innerhalb des Datenblocks, auf den zugegriffen wurde.
	 */
	private int _lastReadPosition;

	/**
	 * Letzter WriteIndex innerhalb des Datenblocks, auf den zugegriffen wurde.
	 */
	private int _lastWritePosition;

// ------------------------------------------------------------------------
// KONSTRUKTOREN  (und vom Konstruktor verwende Methoden)
// ------------------------------------------------------------------------

	/**
	 * Konsturiert ein TlsData Objekt auf Basis der übergebenen tlsBytes[]. Der Zugriff auf die Daten erfolgt gekapselt
	 * über die Klassenmethoden. Das übergebene tlsByte[] wird nicht verändert.
	 *
	 * @param tlsBytes Datenblock gemäß TLS.
	 */
	public TlsData(byte[] tlsBytes) {
		// Byte-Array wird in int-Array mit Masikierung des Vorzeichens umkopiert
		// Dadurch einachers Handling der "Bytes", da diese jetzt ohne Maskierung
		// im Bereich von 0..255 liegen.

		_tlsData = new int[tlsBytes.length];
		for (int i = 0; i < tlsBytes.length; i++) {
			_tlsData[i] = ((int) tlsBytes[i]) & 0xff;
		}

		_currentReadPosition = 0;
		_lastReadPosition = 0;
	}

	/**
	 * Konsturiert ein TlsData Objekt auf Basis der übergebenen Größe. Der Zugriff auf die Daten erfolgt gekapselt über die
	 * Klassenmethoden. Wird über das Ende hinaus geschrieben, wird das Objekt automatisch vergrößert.
	 *
	 * @param length Initiale Länge des TlsData-Objekts.
	 */

	public TlsData(int length) {
		_tlsData = new int[length];
		_currentWritePosition = 0;
		_lastWritePosition = 0;
	}

// ------------------------------------------------------------------------
// KANONISCHE METHODEN
// ------------------------------------------------------------------------

	/**
	 * Gibt den Inhalt als HexDump aus.
	 *
	 * @return HexDump des Inhalts vom TlsData Objekt.
	 */
	public String toString() {
		return HexDumper.toString(HexDumper.toBytes(_tlsData));
	}

// ------------------------------------------------------------------------
// SONSTIGE METHODEN
// ------------------------------------------------------------------------

	/**
	 * Setzt die Leseposition für den nächsten Zugriff auf den Wert vor dem letzten Zugriff.
	 */
	public void backStepReadPosition() {
		_currentReadPosition = _lastReadPosition;
	}

	/**
	 * Gibt die aktuelle Leseposition zurück.
	 *
	 * @return Aktuelle Leseposition.
	 */
	public int currentReadPosition() {
		return _currentReadPosition;
	}

	/**
	 * Gibt die aktuelle Schreibposition zurück.
	 *
	 * @return Aktuelle Schreibposition.
	 */
	public int currentWritePosition() {
		return _currentWritePosition;
	}

	/**
	 * Gibt die Leseposition vor dem letzten Zugriff zurück.
	 *
	 * @return Leseposition vor dem letzten Zugriff.
	 */
	public int lastReadPosition() {
		return _lastReadPosition;
	}

	/**
	 * Gibt die Schreibposition vor dem letzten Zugriff zurück.
	 *
	 * @return Schreibposition vor dem letzten Zugriff.
	 */
	public int lastWritePosition() {
		return _lastWritePosition;
	}

	/**
	 * Liest drei TlsByte als ExtraHighByte, HighByte und LowByte und interpretiert diese jeweils als Halbbytes in
	 * Dezimaldarstellung (0..999999).  Liegt der Wert ausserhalb des Wertebereichs, so wird der Wert 0 zurückgegeben.
	 * Setzt die Leseposition um 3 Positionen weiter.
	 *
	 * @return Wert der 3 TlsByte im Bereich von 0..999999.
	 */
	public int readExtraHighHighLowByteAsHalfByteAsInt() {
		int i;

		i = ((_tlsData[_currentReadPosition] >>> 4) * 100000) +
		    ((_tlsData[_currentReadPosition] & 0x0f) * 10000) +
		    ((_tlsData[_currentReadPosition + 1] >>> 4) * 1000) +
		    ((_tlsData[_currentReadPosition + 1] & 0x0f) * 100) +
		    ((_tlsData[_currentReadPosition + 2] >>> 4) * 10) +
		    (_tlsData[_currentReadPosition + 2] & 0x0f);
		updateReadPosition(3);
		return i < 1000000 ? i : 0;
	}

	/**
	 * Verschiebt die Leseposition für den nächste Zugriff um die angegebene Anzahl Bytes.
	 *
	 * @param count Anzahl der Bytes, um die die Leseposition verschoben werden soll.
	 */
	private void updateReadPosition(int count) {
		_lastReadPosition = _currentReadPosition;
		_currentReadPosition += count;
	}

	/**
	 * Liest zwei TlsByte als HighByte und LowByte und interpretiert diese jeweils als Halbbytes in Dezimaldarstellung
	 * (0..9999). Liegt der Wert ausserhalb des Wertebereichs, so wird der Wert 0 zurückgegeben. Setzt den Leseposition um
	 * 2 Positionen weiter. Positionen weiter.
	 *
	 * @return Wert der 2 TlsByte im Bereich von 0..9999.
	 */
	public int readHighLowByteAsHalfByteAsInt() {
		int i;

		i = ((_tlsData[_currentReadPosition] >>> 4) * 1000) +
		    ((_tlsData[_currentReadPosition] & 0x0f) * 100) +
		    ((_tlsData[_currentReadPosition + 1] >>> 4) * 10) +
		    (_tlsData[_currentReadPosition + 1] & 0x0f);
		updateReadPosition(2);
		return i < 10000 ? i : 0;
	}

	/**
	 * Erzeugt einen neues TlsData Objekt, indem es das Byte an der aktuellen Leseposition als Länge eines nachfolgenden
	 * Datenblocks interpretiert und diesen nachfolgenden Datenblock liest und in ein TlsData-Objekt konvertiert.
	 *
	 * @return TlsData-Objekt.
	 */
	public TlsData readNextTlsDataBlock() {
		int count = readUnsignedByteAsInt(); // Länge des zulesenden Blocks
		return readNextTlsDataBlock(count);
	}

	/**
	 * Liest ein TlsByte als vorzeichenlosen Wert (0..255). Setzt den Leseposition 1 Position weiter.
	 *
	 * @return Wert des TlsByte als vorzeichenloser Wert im Bereich von 0..255.
	 */
	public int readUnsignedByteAsInt() {
		int i = _tlsData[_currentReadPosition];
		updateReadPosition(1);
		return i;
	}

	/**
	 * Erzeugt einen neues TlsData Objekt, indem es ab der aktuellen Leseposition die übergebene Anzahl Bytes liest und in
	 * ein TlsData-Objekt konvertiert.
	 *
	 * @param count Anzahl der zu lesenden Byte.
	 *
	 * @return TlsData-Objekt.
	 */
	public TlsData readNextTlsDataBlock(int count) {
		byte[] bytes = new byte[count];

		for (int i = 0; i < count; ++i) {
			bytes[i] = toBytes()[i + _currentReadPosition];
		}

		TlsData tlsData = new TlsData(bytes);
		updateReadPosition(count);
		return tlsData;
	}

	/**
	 * Gibt die Daten, die durch das TlsData-Objekt repräsentiert werden, als byte[] zurück. Ändert den Zustand des
	 * TlsData-Objekts nicht, verändert also auch nicht die Schreib-/Leseposition. Dient z.B. dazu, die Daten als Byte[]
	 * für das Senden als TLS-Telegramm zu erhalten.
	 *
	 * @return TlsData-Objekt Daten als Byte-Array.
	 *
	 * @throws IllegalArgumentException Wenn im zu konvertierenden int-Array Werte außerhalb des Bereich 0-255 vorkommen.
	 */
	public byte[] toBytes() {
		int length = length();
		byte[] bytes = new byte[length];
		for (int i = 0; i < length; ++i) {
			int val = _tlsData[i];
			if ((val < 0) || (val > 255))
				throw new IllegalArgumentException("Wert am Index " +
				                                   i +
				                                   " liegt nicht zwischen 0 und 255 sondern ist " + val);
			bytes[i] = (byte) (val);
		}
		return bytes;
	}

	/**
	 * Gibt die Länge (Anzahl der Bytes) der Daten zurück.
	 *
	 * @return Länge (Anzahl der Bytes) der Daten.
	 */
	public int length() {
		return _tlsData.length;
	}

	/**
	 * Liest ein TlsByte als vorzeichenbehafteten Wert (-128..127). Setzt den Leseposition 1 Position weiter.
	 *
	 * @return Wert des TlsByte als vorzeichenbehafteten Wert (-128..127).
	 */
	public int readSignedByteAsInt() {
		int i = (byte) _tlsData[_currentReadPosition];
		updateReadPosition(1);
		return i;
	}

	/**
	 * Liest zwee TlsByte als als LowByte und HighByte vorzeichenbehaftet (-32640..32639). Setzt den Leseposition 1
	 * Position weiter.
	 *
	 * @return Wert der 2 TlsByte als vorzeichenbehafteten Wert (-32640..32639).
	 */

	public int readSignedLowHighByteAsInt() {
		int i;
		i = (short) (_tlsData[_currentReadPosition] + (_tlsData[_currentReadPosition + 1] << 8));
		updateReadPosition(2);
		return i;
	}

	/**
	 * Liest drei TlsByte als ExtaHighByte, HighByte und LowByte (0..16777215). Setzt den Leseposition 3 Positionen
	 * weiter.
	 *
	 * @return Wert der 3 TlsByte im Bereich von 0..16777215.
	 */
	public int readUnsignedExtraHighHighLowByteAsInt() {
		int i;
		i = (_tlsData[_currentReadPosition] << 16) +
		    (_tlsData[_currentReadPosition + 1] << 8) +
		    _tlsData[_currentReadPosition + 2];
		updateReadPosition(3);
		return i;
	}

	/**
	 * Liest zwei TlsByte als HighByte und LowByte (0..65535). Setzt den Leseposition 2 Positionen weiter.
	 *
	 * @return Wert der 2 TlsByte im Bereich von 0..65535.
	 */
	public int readUnsignedHighLowByteAsInt() {
		int i;
		i = (_tlsData[_currentReadPosition] << 8) + _tlsData[_currentReadPosition + 1];
		updateReadPosition(2);
		return i;
	}

	/**
	 * Liest zwei TlsByte als LowByte und HighByte (0..65535). Setzt den Leseposition 2 Positionen weiter.
	 *
	 * @return Wert der 2 TlsByte im Bereich von 0..65535.
	 */

	public int readUnsignedLowHighByteAsInt() {
		int i;
		i = _tlsData[_currentReadPosition] + (_tlsData[_currentReadPosition + 1] << 8);
		updateReadPosition(2);
		return i;
	}

	/**
	 * Liest drei TlsByte als LowByte, HighByte und ExtrHighByte (0..16777215). Setzt den Leseposition 3 Positionen
	 * weiter.
	 *
	 * @return Wert der 3 TlsByte im Bereich von 0..16777215.
	 */
	public int readUnsignedLowHighExtraHighByteAsInt() {
		int i;
		i = _tlsData[_currentReadPosition] +
		    (_tlsData[_currentReadPosition + 1] << 8) +
		    (_tlsData[_currentReadPosition + 2] << 16);
		updateReadPosition(3);
		return i;
	}

	/**
	 * Liest vier TlsByte als LowByte, HighByte, ExtrHighByte und VeryHighByte. Setzt den Leseposition 4 Positionen
	 * weiter.
	 *
	 * @return Wert der 3 TlsByte im Bereich von 0..16777215.
	 */
	public int readUnsignedLowHighExtraHighVeryHighByteAsInt() {
		int i;
		i = _tlsData[_currentReadPosition] +
		    (_tlsData[_currentReadPosition + 1] << 8) +
		    (_tlsData[_currentReadPosition + 2] << 16) +
		    (_tlsData[_currentReadPosition + 3] << 24);
		updateReadPosition(4);
		return i;
	}

	/**
	 * Liest ein oder zwei TlsByte als LowByte oder LowByte und HighByte (0..32767). Für Werte kleiner 128 Kodierung als 1
	 * - Byte - Wert, sonst als 2 - Byte - Wert. Die Kennzeichnung erfolgt im 7. Bit des 1. Bytes(low Byte) eines Wertes.
	 * Ist dieses gesetzt(1), so ist die Verkehrsstärke als 1 - Byte - Wert codiert und der Wert ermittelt sich als Byte1 
	 * 128. Ist das Bit nicht gesetzt, so ist der Wert als 2 - Byte - Wert codiert und ermittelt sich als Byte1 + Byte2 *
	 * 128.
	 *
	 * @return Wert der 1 oder 2 TlsByte im Bereich von 0..32767.
	 */

	public int readUnsignedLowOrLowHighByteAsInt() {
		int i, j;
		i = _tlsData[_currentReadPosition];
		if (i >= 128) {
			i = i - 128;
			updateReadPosition(1);
		}
		else {
			j = (_tlsData[_currentReadPosition + 1] * 128);
			i = i + j;
			updateReadPosition(2);
		}
		return i;
	}

	/**
	 * Liest vier TlsByte als VeryHighByte, ExtaHighByte, HighByte und LowByte. Setzt den Leseposition 4 Positionen
	 * weiter.
	 *
	 * @return Wert der 4 TlsByte im Bereich.
	 */
	public int readUnsignedVeryHighExtraHighHighLowByteAsInt() {
		int i;
		i =
				(_tlsData[_currentReadPosition] << 24) +
				(_tlsData[_currentReadPosition + 1] << 16) +
				(_tlsData[_currentReadPosition + 2] << 8) +
				_tlsData[_currentReadPosition + 3];
		updateReadPosition(4);
		return i;
	}

	/**
	 * Setzt die Leseposition auf den Anfang.
	 */
	public void rewindReadPosition() {
		_currentReadPosition = 0;
		_lastReadPosition = 0;
	}

	/**
	 * Schreibt ein byte[] ab der aktuellen Schreibposition. Setzt den Schreibposition byte.length Positionen weiter. Wird
	 * über das Ende hinaus geschrieben, wird das Objekt automatisch vergrößert.
	 *
	 * @param data Das zu scheibende byte[].
	 */
	public void writeByteArray(byte[] data) {
		int updateCount = data.length;
		int endWritePosition = _currentWritePosition + updateCount;
		if (endWritePosition > _tlsData.length) {
			// vergrößen !!
			resize(endWritePosition);
		}
		for (int i = 0; i < data.length; ++i) {
			_tlsData[_currentWritePosition + i] = ((int) data[i]) & 0xff;
		}
		updateWritePosition(updateCount);
	}

	private void resize(int length) {
		//Aktuelle Daten umkopieren
		int[] oldTlsData = new int[_tlsData.length];
		for (int i = 0; i < _tlsData.length; i++) {
			oldTlsData[i] = _tlsData[i];
		}

		// Neuen Datenbereich anlegen und alte Werte zurückkopieren
		_tlsData = new int[length];
		for (int i = 0; i < oldTlsData.length; i++) {
			_tlsData[i] = oldTlsData[i];
		}
	}

	/**
	 * Verschiebt die Schreibposition für den nächste Zugriff um die angegebene Anzahl Bytes.
	 *
	 * @param count Anzahl der Bytes, um die die Schreibposition verschoben werden soll.
	 */
	private void updateWritePosition(int count) {
		_lastWritePosition = _currentWritePosition;
		_currentWritePosition += count;
	}

	/**
	 * Schreibt ein TlsByte (0..255). Setzt den Schreibposition 1 Position weiter. Wird über das Ende hinaus geschrieben,
	 * wird das Objekt automatisch vergrößert.
	 *
	 * @param data Der zu konvertierende Wert.
	 */
	public void writeIntAsByte(int data) {
		int updateCount = 1;
		int endWritePosition = _currentWritePosition + updateCount;
		if (endWritePosition > _tlsData.length) {
			// vergrößen !!
			resize(endWritePosition);
		}
		_tlsData[_currentWritePosition] = data;
		updateWritePosition(updateCount);
	}

	/**
	 * Schreibt ein TlsByte (0..255) an die angegebene Position. Verändert weder Lese- noch Schreibposition.
	 *
	 * @param data  Der zu konvertierende Wert.
	 * @param index Index des TLSBytes, welcher geschrieben werden soll.
	 */
	public void writeIntAsByte(int data, int index) {
		_tlsData[index] = data;
	}

	/**
	 * Schreibt drei "TlsByte" als ExtrHighByte, HighByte und LowByte (0..16777215). Setzt den Schreibposition 3 Positionen
	 * weiter. Wird über das Ende hinaus geschrieben, wird das Objekt automatisch vergrößert.
	 *
	 * @param data Der zu konvertierende Wert.
	 */
	public void writeIntAsExtraHighHighLowByte(int data) {
		int updateCount = 3;
		int endWritePosition = _currentWritePosition + updateCount;
		if (endWritePosition > _tlsData.length) {
			// vergrößen !!
			resize(endWritePosition);
		}
		_tlsData[_currentWritePosition] = (data & 0xff0000) >>> 16;
		_tlsData[_currentWritePosition + 1] = (data & 0xff00) >>> 8;
		_tlsData[_currentWritePosition + 2] = data & 0xff;
		updateWritePosition(updateCount);
	}

	/**
	 * Schreibt drei "TlsByte" als ExtraHighByte, HighByte und LowByte und interpretiert diese jeweils als Halbbytes in
	 * Dezimaldarstellung (0..999999). Setzt den Schreibposition 3 Positionen weiter. Wird über das Ende hinaus
	 * geschrieben, wird das Objekt automatisch vergrößert.
	 *
	 * @param data Der zu konvertierende Wert.
	 */
	public void writeIntAsExtraHighHighLowByteAsHalfByte(int data) {
		int updateCount = 3;
		int endWritePosition = _currentWritePosition + updateCount;
		if (endWritePosition > _tlsData.length) {
			// vergrößen !!
			resize(endWritePosition);
		}
		int digit100000;
		int digit10000;
		int digit1000;
		int digit100;
		int digit10;
		int digit1;

		digit100000 = data / 100000;
		data = data % 100000;
		digit10000 = data / 10000;
		data = data % 10000;
		digit1000 = data / 1000;
		data = data % 1000;
		digit100 = data / 100;
		data = data % 100;
		digit10 = data / 10;
		data = data % 10;
		digit1 = data;

		_tlsData[_currentWritePosition] = (digit100000 << 4) + digit10000;
		_tlsData[_currentWritePosition + 1] = (digit1000 << 4) + digit100;
		_tlsData[_currentWritePosition + 2] = (digit10 << 4) + digit1;

		updateWritePosition(updateCount);
	}

	/**
	 * Schreibt zwei "TlsByte" als HighLowByte und LowByte (0..65535). Setzt den Schreibposition 2 Positionen weiter. Wird
	 * über das Ende hinaus geschrieben, wird das Objekt automatisch vergrößert.
	 *
	 * @param data Der zu konvertierende Wert.
	 */
	public void writeIntAsHighLowByte(int data) {
		int updateCount = 2;
		int endWritePosition = _currentWritePosition + updateCount;
		if (endWritePosition > _tlsData.length) {
			// vergrößen !!
			resize(endWritePosition);
		}
		_tlsData[_currentWritePosition] = (data & 0xff00) >>> 8;
		_tlsData[_currentWritePosition + 1] = data & 0xff;
		updateWritePosition(updateCount);
	}

	/**
	 * Schreibt zwei "TlsByte" als HighByte und LowByte und interpretiert diese jeweils als Halbbytes in Dezimaldarstellung
	 * (0..9999). Setzt den Schreibposition 2 Positionen weiter. Wird über das Ende hinaus geschrieben, wird das Objekt
	 * automatisch vergrößert.
	 *
	 * @param data Der zu konvertierende Wert.
	 */
	public void writeIntAsHighLowByteAsHalfByte(int data) {
		int updateCount = 2;
		int endWritePosition = _currentWritePosition + updateCount;
		if (endWritePosition > _tlsData.length) {
			// vergrößen !!
			resize(endWritePosition);
		}
		int digit1000;
		int digit100;
		int digit10;
		int digit1;

		digit1000 = data / 1000;
		data = data % 1000;
		digit100 = data / 100;
		data = data % 100;
		digit10 = data / 10;
		data = data % 10;
		digit1 = data;

		_tlsData[_currentWritePosition] = (digit1000 << 4) + digit100;
		_tlsData[_currentWritePosition + 1] = (digit10 << 4) + digit1;
		updateWritePosition(updateCount);
	}

	/**
	 * Schreibt zwei "TlsByte" als LowByte und HighByte (0..65535). Setzt den Schreibposition 2 Positionen weiter. Wird
	 * über das Ende hinaus geschrieben, wird das Objekt automatisch vergrößert.
	 *
	 * @param data Der zu konvertierende Wert.
	 */
	public void writeIntAsLowHighByte(int data) {
		int updateCount = 2;
		int endWritePosition = _currentWritePosition + updateCount;
		if (endWritePosition > _tlsData.length) {
			// vergrößen !!
			resize(endWritePosition);
		}
		_tlsData[_currentWritePosition] = data & 0xff;
		_tlsData[_currentWritePosition + 1] = (data & 0xff00) >>> 8;
		updateWritePosition(updateCount);
	}

	/**
	 * Schreibt drei "TlsByte" als LowByte, HighByte und ExtrHighByte (0..16777215). Setzt den Schreibposition 3 Positionen
	 * weiter. Wird über das Ende hinaus geschrieben, wird das Objekt automatisch vergrößert.
	 *
	 * @param data Der zu konvertierende Wert.
	 */
	public void writeIntAsLowHighExtraHighByte(int data) {
		int updateCount = 3;
		int endWritePosition = _currentWritePosition + updateCount;
		if (endWritePosition > _tlsData.length) {
			// vergrößen !!
			resize(endWritePosition);
		}
		_tlsData[_currentWritePosition] = data & 0xff;
		_tlsData[_currentWritePosition + 1] = (data & 0xff00) >>> 8;
		_tlsData[_currentWritePosition + 2] = (data & 0xff0000) >>> 16;

		updateWritePosition(updateCount);
	}

	/**
	 * Schreibt ein TlsData-Objekt ab der aktuellen Schreibposition. Setzt den Schreibposition data.length Positionen
	 * weiter. Wird über das Ende hinaus geschrieben, wird das Objekt automatisch vergrößert.
	 *
	 * @param data Das zu scheibende byte[].
	 */
	public void writeTlsData(TlsData data) {
		int updateCount = data.length();
		int endWritePosition = _currentWritePosition + updateCount;
		if (endWritePosition > _tlsData.length) {
			// vergrößen !!
			resize(endWritePosition);
		}
		for (int i = 0; i < data.length(); ++i) {
			_tlsData[_currentWritePosition + i] = data.readUnsignedByteAsInt();
		}
		updateWritePosition(updateCount);
	}
}
